package org.bitbucket.zetro.squaredino.backend;

/**
 * All instances of this Cell class will count as living cells but can have
 * different states. All cells in the same state is equal to each other.
 */
public class Cell {
	/**
	 * Cell state.
	 */
	public final int state;

	private Cell(int state) {
		this.state = state;
	}

	/**
	 * Creates a new cell in a given state. State
	 * <code>0</code> is a dead cell and returns
	 * <code>null</code>.
	 *
	 * @param state the state
	 * @return a new cell in the given state
	 */
	public static Cell createCell(int state) {
		if (state == 0) {
			return null;
		}

		return new Cell(state);
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Cell other = (Cell) obj;
		if (this.state != other.state) {
			return false;
		}
		return true;
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = 71 * hash + this.state;
		return hash;
	}

	@Override
	public String toString() {
		return "Cell{" + "state=" + state + '}';
	}
}
