package org.bitbucket.zetro.squaredino.backend;

import java.awt.Color;
import java.util.Arrays;
import java.util.List;

/**
 * Implementation of HPP. <br/> Example:
 * <code>14P$P12.P$P12.P$P8.A3.P$P12.P$P12.P$P3.M8.P$P12.P$P12.P$14P!</code>
 */
public class HPP extends CellularAutomaton {
	@Override
	public List<Cell> getStates() {
		return Arrays.asList(Cell.createCell(2),
				Cell.createCell(4),
				Cell.createCell(8),
				Cell.createCell(1),
				Cell.createCell(16),
				null);
	}

	@Override
	public int getColor(Cell cell) {
		if (cell == null) {
			return super.getColor(cell);
		}

		int state = cell.state;
		boolean isMirror = (state >>> 4) == 1;
		int density = ((state & (1 << 3)) != 0 ? 1 : 0)
				+ ((state & (1 << 2)) != 0 ? 1 : 0)
				+ ((state & (1 << 1)) != 0 ? 1 : 0)
				+ ((state & 1) != 0 ? 1 : 0);
		int c = 255 / 4 * density;
		return new Color(c / (isMirror ? 2 : 1), c, c).getRGB();
	}

	@Override
	public Cell getNextState(Cell cell, Cell... neighbours) {
		boolean isMirror = ((cell != null ? cell.state : 0) >>> 4) == 1;
		int collectedState = isMirror ? 1 << 4 : 0;

		// Collect any orthogonally neighbouring particles moving to this cell.
		for (int i = 0; i < neighbours.length; i += 2) {
			Cell neighbour = neighbours[i];
			if (neighbour == null) {
				continue;
			}

			int direction = (i / 2 + 2 + 1) % 4; // '/2' due to orthogonal neighbour, '+2' is opposite direction and '+1' is to get compliant with SENW
			if ((neighbour.state & (1 << direction)) != 0) {
				if (isMirror) {
					direction = (direction + 2) % 4;
				}
				collectedState |= 1 << direction;
			}
		}

		// Handle head-on collisions
		if (collectedState == 5) { // EW collision -> NS
			collectedState = 10;
		} else if (collectedState == 10) { // SN collision -> WE
			collectedState = 5;
		}

		// Create cell
		return Cell.createCell(collectedState);
	}

	@Override
	public String getRule() {
		return null;
	}
}
