package org.bitbucket.zetro.squaredino.backend.experimental;

/**
 * An immutable quadnode that stores a value. hashcode() depends on the stored
 * value.
 *
 * @param <T> data type of the stored value
 */
public class HashContentNode<T> extends HashNode {
	/**
	 * Stored value.
	 */
	public final T content;

	/**
	 * ContentHashNode constructor.
	 *
	 * @param content stored value
	 */
	private HashContentNode(T content) {
		super(null, null, null, null);

		this.content = content;
	}

	/**
	 * Creates a new content node.
	 *
	 * @param <T> data type of content
	 * @param cache node cache
	 * @param content the content
	 * @return the new node
	 */
	public static <T> HashNode create(HashNodeCache cache, T content) {
		return cache.findOrCacheNode(new HashContentNode<T>(content));
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		@SuppressWarnings("unchecked")
		final HashContentNode<T> other = (HashContentNode<T>) obj;
		if (this.content != other.content && (this.content == null || !this.content.equals(other.content))) {
			return false;
		}
		return true;
	}

	@Override
	public int hashCode() {
		int hash = 3;
		hash = 79 * hash + (this.content != null ? this.content.hashCode() : 0);
		return hash;
	}

	@Override
	public int size() {
		return this.content != null ? 1 : 0;
	}

	@Override
	public String toString() {
		return "HashContentNode{" + "content=" + content + '}';
	}
}
