package org.bitbucket.zetro.squaredino.backend.experimental;

import java.util.HashMap;

/**
 * HashNodeCache class.
 */
public class HashNodeCache {
	private final HashMap<HashNode, HashNode> cache;

	/**
	 * HashNodeCache constructor.
	 */
	public HashNodeCache() {
		cache = new HashMap<HashNode, HashNode>();
	}

	/**
	 * Returns an already cached and equal node or caches the given node and
	 * return the given node.
	 *
	 * @param n the given node
	 * @return an already cached and equal node or the given node
	 */
	public HashNode findOrCacheNode(HashNode n) {
		HashNode cachedNode = cache.get(n);
		if (cachedNode == null) {
			cache.put(n, n);
			cachedNode = n;
		}
		return cachedNode;
	}

	/**
	 * Returns <code>true</code> if <code>node</code> is cached.
	 * 
	 * @param node
	 * @return <code>true</code> if <code>node</code> is cached
	 */
	public boolean isCached(HashNode node) {
		return cache.containsKey(node);
	}

	/**
	 * Removes all cached nodes. The cache will be empty after this call
	 * returns.
	 */
	public void clear() {
		cache.clear();
	}

	/**
	 * Returns the size of this cache.
	 *
	 * @return the size of this cache
	 */
	public int size() {
		return cache.size();
	}
}
