package org.bitbucket.zetro.squaredino.gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.image.BufferedImage;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JPanel;
import org.bitbucket.zetro.squaredino.backend.Board;
import org.bitbucket.zetro.squaredino.backend.BoardFactory;
import org.bitbucket.zetro.squaredino.backend.Cell;
import org.bitbucket.zetro.squaredino.misc.Point;
import org.bitbucket.zetro.squaredino.misc.Selection;


public class BoardPanel extends JPanel {
	private static final long serialVersionUID = 4361849887580197351L;

	private Board board;
	
	private int width, height; //number of cells
	
	private int paddx; 
	private int paddy; 
	private int pressX; 
	private int pressY; 
	private int tempaddx; 
	private int tempaddy; 
	
	private double zoom = 16; //width and height of a cell in pixels
	private boolean drawGrid = true;
	
	private boolean isSelecting = false; 
	private boolean movesSelection = false; 
	private Selection selected;
	private Board selection; 
	private Point tempSelmove; 
	
	private boolean editable = true;
	private boolean rightClicking = false; 
	private Cell paintCell; 
	
	private boolean copySelection = false;
	private boolean moveDead = true;
	
	public BoardPanel() {
		this(0, 0); //Är det såhär man gör? 
	}
	public BoardPanel(int w, int h) {
		width = w; 
		height = h; 
		board = BoardFactory.createBoard(width, height); 
		
		paddx = paddy = (int) (4*zoom); 
		
		addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				copySelection = e.isControlDown();
				moveDead = !e.isAltDown();
				repaint();
			}

			@Override
			public void keyReleased(KeyEvent e) {
				copySelection = e.isControlDown();
				moveDead = !e.isAltDown();
				repaint();
			}
		});
		addMouseMotionListener(new MouseAdapter() {
			public void mouseDragged(MouseEvent e) {
				Point p = calPos(e.getX() - tempaddx, e.getY() - tempaddy);
				if(rightClicking) {
					setCell(paintCell, p.x, p.y);
				}
				else {
					if(isSelecting) {
						selected.setLast(calPos(e.getX(), e.getY())); 
					}
					else if(selected != null && movesSelection) {
						Point press = calPos(pressX, pressY); 
						tempSelmove = new Point(p.x - press.x, p.y - press.y); 
					}
					else {
						tempaddx = e.getX() - pressX; 
						tempaddy = e.getY() - pressY; 
					}
				}
				repaint(); 
			}
		});
		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				pressX = e.getX();
				pressY = e.getY(); 
				Point p = calPos(e.getX()-tempaddx, e.getY()-tempaddy);
				/*
				 * Pressed the right mousebutton and draws live/dead cells. 
				 */
				if(e.getButton() == MouseEvent.BUTTON3) { 
					rightClicking = true; 
					paintCell = getPaintCell(p.x, p.y);
					setCell(paintCell, p.x, p.y); 
				}
				/*
				 * Pressed something else (typically left mousebutton)
				 */
				else {
					if(isSelecting) { //starts drawing selection if in selection mode
						selected = new Selection(calPos(e.getX(), e.getY())); 
					}
					else if(selected != null && selected.contains(p)) {
						movesSelection = true; 
						selection = board.getSubBoard(selected).copy(); 
						if (!copySelection)
							board.clear(selected);
					}
					else {
					}
				}
				repaint(); 
			}
			public void mouseReleased(MouseEvent e) {
				if(e.getButton() == MouseEvent.BUTTON3) {rightClicking = false;}
				else {
					if(isSelecting) {
						isSelecting = false; 
					}
				}
				paddx += tempaddx; 
				paddy += tempaddy; 
				
				if(selected != null && selection != null) {
					if(tempSelmove != null) {
						selected.moveX(tempSelmove.x); 
						selected.moveY(tempSelmove.y);
						if (moveDead)
							board.clear(selected);
						board.insertBoard(selection, tempSelmove.x, tempSelmove.y); 
					} else {
						if (moveDead)
							board.clear(selected);
						board.insertBoard(selection, 0, 0); 
					}
					selection = null;
					tempSelmove = null; 
				}
				tempaddx = tempaddy = 0; 
				movesSelection = false; 
				
				repaint(); 
			}
			public void mouseClicked(MouseEvent e) {
				if(e.getButton() == MouseEvent.BUTTON3) {}
				else {
					if(!isSelecting) {
						Point p = calPos(e.getX(), e.getY()); 
						paintCell = getPaintCell(p.x, p.y); 
						setCell(paintCell, p.x, p.y); 
						System.out.println(p.x + ":" + p.y);
						repaint(); 
					}
				}
			}
		});
		
		addMouseWheelListener(new MouseWheelListener() {
			private int zoomLevel = (int) (zoom / 2);
			
			@Override
			public void mouseWheelMoved(MouseWheelEvent e) {
				zoomLevel -= e.getWheelRotation();
				double newZoom;
				if (zoomLevel >= 0) {
					newZoom = Math.max(1, 2 * zoomLevel);
				} else if (zoomLevel == -1) {
					newZoom = 0.5;
				} else {
					newZoom = 1.0 / (zoomLevel * zoomLevel);
				}
				double diff = zoom - newZoom;

				paddx += diff * (e.getX() - paddx)/zoom;
				paddy += diff * (e.getY() - paddy)/zoom; 

				zoom = newZoom; 
				repaint(); 
			}
		});
	}

	private Cell getPaintCell(int x, int y) {
		if (editable) {
			List<Cell> states = board.getStates();
			int idx = states.indexOf(board.getCell(x, y));
			return states.get((idx + 1) % states.size());
		}
		return null;
	}
	
	private void setCell(Cell cell, int x, int y) {
		if (editable) {
			if (paintCell == null) {
				board.removeCell(x, y);
			} else {
				board.addCell(cell, x, y);
			}
		}
	}
	
	public Board getBoard() {
		return board; 
	}
	public void step() {
		board.step(); 
		repaint(); 
	}
	public void clear() {
		if(selected != null) {
			board.clear(selected); 
		}
		else 
			board.clear(); 
		repaint(); 
	}
	public Point calPos(int mouseX, int mouseY) {
		double x = (mouseX - paddx); 
		double y = (mouseY - paddy);
		
		if(x < 0) { //Stupid negative axis bug. 
			x /= zoom;
			x -= 1; 
		}
		else
			x /= zoom; 
		if(y < 0) {
			y /= zoom; 
			y -= 1;
		}
		else
			y /= zoom; 
//		System.out.println(x + ":" + y); 
		return new Point((int) x, (int) y); 
	}
	public void setSelectMode(boolean b) {
		isSelecting = b; 
		selected = null;
		selection = null; 
		tempSelmove = null; 
		repaint(); 
	}
	public void setGrid(boolean b) {
		drawGrid = b; 
		repaint(); 
	}
	public void setBoard(Board board) {
		this.board = board;
		paddx = (int) (getWidth() - zoom) / 2;
		paddy = (int) (getHeight() - zoom) / 2;
		repaint();
	}
	public Board getSelectedBoard() {
		if(selected != null) 
			return board.getSubBoard(selected); 
		else 
			return board; 
	}

	public void setEditable(boolean b) {
		editable = b;
	}
	
	public void paintComponent(Graphics g) {
		g.setColor(Color.WHITE); 
		g.fillRect(0, 0, getWidth(), getHeight());
		
		/* Draws cells */
		g.setColor(Color.BLACK);
		Point topLeft = calPos(-tempaddx, -tempaddy);
		Selection view = new Selection(topLeft,
				new Point(topLeft.x + (int) (getWidth() / zoom) + 1, 
						  topLeft.y + (int) (getHeight() / zoom) + 1));
		int ignoreLevel = zoom < 1 ? (int) (Math.log(1 / zoom) / Math.log(2)) : 0;
		Iterator<Map.Entry<Point, Cell>> cells = board.getLivingCells(view, ignoreLevel); 
		int size = (int) Math.max(1, zoom);
		if (size == 1) {
			int w = getWidth();
			int h = getHeight();
			BufferedImage image = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
			
			int defaultColor = board.getColor(Cell.createCell(1));
			while(cells.hasNext()) {
				Map.Entry<Point, Cell> entry = cells.next();
				Point p = entry.getKey(); 
				int x = paddx + tempaddx + (int) (p.x * zoom);
				int y = paddy + tempaddy + (int) (p.y * zoom);
				if (x >= 0 && y >= 0 && x < w && y < h) {
					image.setRGB(x, y, ignoreLevel == 0
							? board.getColor(entry.getValue()) : defaultColor);
				}
			}
			
			g.drawImage(image, 0, 0, null);
		} else {
			while(cells.hasNext()) {
				Map.Entry<Point, Cell> entry = cells.next();
				Point p = entry.getKey(); 
				g.setColor(new Color(board.getColor(entry.getValue())));
				g.fillRect(paddx + tempaddx + (int) (p.x * zoom), 
							paddy + tempaddy + (int) (p.y * zoom), 
							size, size
				); 
			}
		}
		
		/* Draws selection */
		g.setColor(new Color(32, 255, 32, 128)); 
		if(selected != null) {
			Point p = tempSelmove==null?new Point(0, 0):tempSelmove;
			topLeft = calPos((int)(-p.x*zoom), (int) (-p.y*zoom));
			view = new Selection(topLeft,
					new Point(topLeft.x + (int) (getWidth() / zoom) + 1, 
							topLeft.y + (int) (getHeight() / zoom) + 1));
			
			if(selection != null && movesSelection) {
				if (moveDead) {
					g.setColor(Color.WHITE); 
					g.fillRect(paddx + tempaddx + (int) ((p.x + selected.getTopLeft().x) * zoom), 
							paddy + tempaddy + (int) ((p.y + selected.getTopLeft().y) * zoom), 
							(int) ((selected.getWidth()+1)*zoom), (int) ((selected.getHeight()+1)*zoom)
					);
				}
				g.setColor(Color.BLACK);
				int defaultColor = board.getColor(Cell.createCell(1));
				Iterator<Map.Entry<Point, Cell>> cells2 = selection.getLivingCells(view, ignoreLevel); 
				while(cells2.hasNext()) {
					Map.Entry<Point, Cell> entry = cells2.next();
					Point p2 = entry.getKey(); 
					g.setColor(new Color(ignoreLevel == 0
							? board.getColor(entry.getValue()) : defaultColor));
					g.fillRect(paddx + tempaddx + (int) ((p.x + p2.x) * zoom), 
								paddy + tempaddy + (int) ((p.y + p2.y) * zoom), 
								size, size
					); 
				}
			}
			
			/* Draws green background */
			g.setColor(new Color(32, 255, 32, 128)); 
			g.fillRect(paddx + tempaddx + (int) ((p.x + selected.getTopLeft().x) * zoom), 
					paddy + tempaddy + (int) ((p.y + selected.getTopLeft().y) * zoom), 
					(int) ((selected.getWidth()+1)*zoom), (int) ((selected.getHeight()+1)*zoom)
			); 
		}
		
		/* Draws grid */
		g.setColor(Color.GRAY);
		if(drawGrid && size > 1) {
			for(int x=(paddx+tempaddx)%size - size;x<getWidth();x+=size) {
				g.drawLine(x, 0, x, getHeight()); 
			}
			for(int y=(paddy+tempaddy)%size - size;y<getHeight();y+=size) {
				g.drawLine(0, y, getWidth(), y); 
			}
		}
	}
}
