package org.bitbucket.zetro.squaredino.gui;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JToolBar;
import javax.swing.Timer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.bitbucket.zetro.squaredino.backend.Board;
import org.bitbucket.zetro.squaredino.backend.BoardFactory;
import org.bitbucket.zetro.squaredino.io.BoardData;

import org.bitbucket.zetro.squaredino.misc.Misc;

//class TestP extends JPanel {
//	public void paintComponent(Graphics g) {
//		g.drawRect(0, 0, getWidth(), getHeight());
//	}
//}

public class ControlBar extends JToolBar { 
	private static final long serialVersionUID = -4596851409109069705L;
	
	private BoardPanel boardPanel; 
	private GraphPanel graph; 
	
	private JButton play; 
	private ImageIcon playIcon = new ImageIcon("images/play.png"); 
	private ImageIcon playPressed; 
	private ImageIcon pauseIcon = new ImageIcon("images/pause.png");
	private ImageIcon pausePressed; 
	private JLabel playlabel;
	private boolean isPlaying = false; 
	
	private Timer player; 
	
	private JButton step; 
	private JButton clear; 
	private JButton select; 
	private JButton deselect;
	private JButton rule; 
	private JButton importt; 
	private JButton export; 

	private JSlider slider; 
	
	private JCheckBox grid; 
	private JCheckBox graphbox; 
	
	public ControlBar(final JFrame owner, BoardPanel bp, GraphPanel gp) {
		boardPanel = bp; 
		graph = gp; 
		
		play = new JButton(); 
		play.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				isPlaying = !isPlaying; 
				if(isPlaying) {
					play.setIcon(pauseIcon); 
					play.setPressedIcon(pausePressed); 
					playlabel.setText("Pause"); 
					player.start(); 
				}
				else {
					play.setIcon(playIcon); 
					play.setPressedIcon(playPressed); 
					playlabel.setText("Play"); 
					player.stop(); 
				}
			}
		});
		playlabel = new JLabel("Play"); 
		player = new Timer(75, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				boardPanel.step(); 
				if(graphbox.isSelected()) 
					graph.addValue(boardPanel.getBoard().getNumberOfCells()); 
			}
		}); 
		add(prepareButton(play, playIcon, "Play")); 
		
		step = new JButton(); 
		step.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				boardPanel.step(); 
				if(graphbox.isSelected()) 
					graph.addValue(boardPanel.getBoard().getNumberOfCells()); 
			}
		});
		add(prepareButton(step, new ImageIcon("images/step.png"), "Step")); 
		
		clear = new JButton(); 
		clear.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				boardPanel.clear(); 
				graph.clear(); 
			}
		});
		add(prepareButton(clear, new ImageIcon("images/clear.png"), "Clear")); 
		
		addSeparator(new Dimension(16, 16));
		
		select = new JButton(); 
		select.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				System.out.println("Select!"); 
				boardPanel.setSelectMode(true); 
			}
		});
		add(prepareButton(select, new ImageIcon("images/select.png"), "Select")); 
		
		deselect = new JButton(); 
		deselect.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				System.out.println("Deselect"); 
				boardPanel.setSelectMode(false); 
			}
		});
		add(prepareButton(deselect, new ImageIcon("images/deselect.png"), "Deselect")); 
		
		addSeparator(new Dimension(16, 32));
		
		rule = new JButton(); 
		rule.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.out.println("Rule"); 
				String currentRule = boardPanel.getBoard().getRuleAsString();
				Object result = JOptionPane.showInputDialog(boardPanel, "New rule:", "Change rule", JOptionPane.PLAIN_MESSAGE, null, null, currentRule); 
				if (result != null)
					try {
						boardPanel.getBoard().setRule(result.toString()); 
					}catch(IllegalArgumentException ae) {
						JOptionPane.showMessageDialog(ControlBar.this, "Error: Invalid rule"); 
					}
			}
		});
		add(prepareButton(rule, new ImageIcon("images/rule.png"), "Rule"));
		
		importt = new JButton(); 
		importt.addActionListener(new ActionListener() {
			private File currentDirectory = new File("./patterns");
			
			@Override
			public void actionPerformed(ActionEvent e) {
				System.out.println("Import!"); 
				ImportDialog importDialog = new ImportDialog(owner, currentDirectory); 
				currentDirectory = importDialog.getCurrentDirectory();
				BoardData boardData = importDialog.getBoardData();
				importDialog.clear();
				if (boardData != null) {
					Board board = boardData.getBoard();
					if (board != null) {
						boardPanel.setBoard(board);
					} else {
						JOptionPane.showMessageDialog(ControlBar.this, "No board imported"); 
					}
				}
			}
		});
		add(prepareButton(importt, new ImageIcon("images/import.png"), "Import")); 
		
		export = new JButton(); 
		export.addActionListener(new ActionListener() {
			private File currentDirectory = new File("./patterns");
			
			@Override
			public void actionPerformed(ActionEvent e) {
				System.out.println("Export!"); 
				ExportDialog exportDialog = new ExportDialog(owner, BoardFactory.unmodifiableBoard(boardPanel.getSelectedBoard()), currentDirectory); 
				currentDirectory = exportDialog.getCurrentDirectory();
			}
		});
		add(prepareButton(export, new ImageIcon("images/export2.png"), "Export")); 
		
		addSeparator(new Dimension(16, 16)); 
		
		grid = new JCheckBox("Grid", false);
		prepareBox(grid);
		grid.setSelected(true); 
		grid.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				boardPanel.setGrid(grid.isSelected()); 	
			}
		});
		graphbox = new JCheckBox("Graph", true);
		prepareBox(graphbox); 
		graphbox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub	
				graph.setVisible(graphbox.isSelected()); 
			}
		});
		
		Box b = Box.createVerticalBox(); 
		b.add(grid); 
		b.add(graphbox);
		b.setAlignmentX(JComponent.CENTER_ALIGNMENT); 
		
		//Had to wrap it because it looked weird when toolbar was on the side. 
		Box bb = Box.createVerticalBox(); 
		bb.add(b); 
		add(bb); 
		
		addSeparator(new Dimension(16, 16));
		
		slider = new JSlider(5, 500, 425); 
		slider.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				player.setDelay(500 - slider.getValue()); 	
			}
		});
		slider.setOpaque(false);
		slider.setMaximumSize(new Dimension(96, 40)); 
		slider.setPreferredSize(new Dimension(96, 40)); 
		slider.setFocusable(false); 
		
		add(slider); 
	}
	private JPanel prepareButton(JButton b, ImageIcon i, String t) {
		b.setBorder(null); 
		b.setMaximumSize(new Dimension(32, 32)); 
		b.setIcon(i); 
		b.setPressedIcon(Misc.pressedButtonIcon(i)); 
		b.setContentAreaFilled(false); 
		b.setFocusPainted(false); 
		b.setAlignmentX(JComponent.CENTER_ALIGNMENT); 
		b.setFocusable(false);
		
		Box box = Box.createVerticalBox(); 
		box.add(b); 
		
		JLabel j = new JLabel(t); 
		if(t.equals("Play")) {
			j = playlabel;
			playPressed = (ImageIcon) b.getPressedIcon(); 
			pausePressed = Misc.pressedButtonIcon(pauseIcon); 
		}
		j.setFont(new Font("DejaVu", Font.PLAIN, 11)); 
		j.setAlignmentX(JComponent.CENTER_ALIGNMENT); 
		box.add(j); 
		
		JPanel p = new JPanel();
		p.setOpaque(false); 
		p.setMaximumSize(new Dimension(48, 60)); 
		p.setMinimumSize(new Dimension(48, 60)); 
		p.add(box); 
		
		return p;
	}
	public void prepareBox(final JCheckBox box) {
		box.setOpaque(false); 
		box.setFont(new Font("DejaVu", Font.PLAIN, 11)); 
		box.setFocusable(false); 
		box.setIcon(new ImageIcon("images/checkbox.png")); 
		box.setPressedIcon(new ImageIcon("images/checkbox-pressed.png")); 
		box.setSelectedIcon(new ImageIcon("images/checkbox-pressed.png")); 
		box.setAlignmentX(JComponent.LEFT_ALIGNMENT); 
		
		box.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				if(box.isSelected()) 
					box.setPressedIcon(new ImageIcon("images/checkbox.png")); 
				else
					box.setPressedIcon(new ImageIcon("images/checkbox-pressed.png"));
			}
		});
	}
}
