package org.bitbucket.zetro.squaredino.gui;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.List;
import javax.swing.BorderFactory;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import org.bitbucket.zetro.squaredino.backend.Board;
import org.bitbucket.zetro.squaredino.backend.BoardImpl;
import org.bitbucket.zetro.squaredino.backend.GameOfLife;
import org.bitbucket.zetro.squaredino.io.BoardData;
import org.bitbucket.zetro.squaredino.io.SquareIO;

public class ExportDialog extends JDialog {
	private static final long serialVersionUID = -6443441330576255898L;
	
	private File currentDirectory;
	
	private BoardPanel boardPanel;
	
	private final JTextField name;
	private final JTextField creator;
	private final JTextField rule;
	private final JLabel width;
	private final JTextArea comments;
	
	private JButton exportToText;
	private JButton exportToFile;
	
	public ExportDialog(final JFrame owner, final Board b, File currentDirectory) {
		super(owner, true);
		
		this.currentDirectory = currentDirectory;
		
		setTitle("Export");
		setSize(500, 600);
		setResizable(false); 
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setLocationRelativeTo(null);


		name = new JTextField();
		creator = new JTextField();
		rule = new JTextField();
		width = new JLabel("?");
		width.setFont(new Font("DejaVu", Font.PLAIN, 11));
		comments = new JTextArea();
		comments.setLineWrap(true);
		comments.setBorder(LineBorder.createGrayLineBorder());

		JPanel top = new JPanel();
		top.setLayout(new BoxLayout(top, BoxLayout.PAGE_AXIS));
		boardPanel = new BoardPanel();
		boardPanel.setEditable(false);
		boardPanel.setBoard(b);
		boardPanel.setPreferredSize(new Dimension(200, 200));
		top.add(boardPanel);

		JPanel topFields = new JPanel();
		topFields.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		topFields.setPreferredSize(new Dimension(200, 200));
		topFields.setLayout(new BoxLayout(topFields, BoxLayout.PAGE_AXIS));
		topFields.add(Box.createVerticalGlue());
		
		JComponent metadataBox = new JPanel(new GridBagLayout());

		addTextField(metadataBox, "Name: ", name, 0, false);
		addTextField(metadataBox, "Creator: ", creator, 1, false);
		addTextField(metadataBox, "Rule: ", rule, 2, false);
		addTextField(metadataBox, "Width/Height: ", width, 3, false);
		addTextField(metadataBox, "Comments: ", comments, 4, true);

		topFields.add(metadataBox);
		topFields.add(Box.createVerticalGlue());

		exportToFile = new JButton("Export to file");
		exportToText = new JButton("Export to text");
		JPanel middleButtons = new JPanel();
		middleButtons.setLayout(new BoxLayout(middleButtons, BoxLayout.LINE_AXIS));
		middleButtons.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		middleButtons.add(Box.createHorizontalGlue());
		middleButtons.add(exportToText);
		middleButtons.add(Box.createHorizontalStrut(16));
		middleButtons.add(exportToFile);
		middleButtons.add(Box.createHorizontalGlue());

		top.add(topFields);
		top.add(middleButtons);
		add(top);
		
		addListeners();
		initValues();

		setVisible(true);
	}

	private void addTextField(JComponent container, String title, final JComponent f, int row, boolean maxSize) {
			JLabel l = new JLabel(title);
			l.setFont(new Font("DejaVu", Font.PLAIN, 11));
			
			GridBagConstraints lc = new GridBagConstraints();
			lc.anchor = GridBagConstraints.LINE_START;
			lc.gridx = 0;
			lc.gridy = row;
			lc.insets = new Insets(0, 4, 4, 10);
			if (maxSize) {
				lc.weighty = 1;
			}
			container.add(l,lc);
			
			GridBagConstraints cc = new GridBagConstraints();
			cc.fill = GridBagConstraints.HORIZONTAL;
			cc.weightx = 1;
			cc.gridx = 1;
			cc.gridy = row;
			cc.insets = new Insets(0, 4, 4, 0);
			if (maxSize) {
				cc.fill = GridBagConstraints.BOTH;
				cc.weighty = 1;
			}
			container.add(f,cc);
	}

	private void initValues() {
		rule.setText(boardPanel.getBoard().getRuleAsString());
		BoardData boardData = getBoardData();
		String sw = boardData.getWidth() == Board.INFINITE ? "-" : String.valueOf(boardData.getWidth());
		String sh = boardData.getHeight() == Board.INFINITE ? "-" : String.valueOf(boardData.getHeight());
		width.setText(sw + "/" + sh);
	}

	private void addListeners() {
		exportToText.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				BoardData boardData = getBoardData();
				StringWriter writer = new StringWriter();
				try {
					SquareIO.save(boardData, writer);
					JDialog g = new JDialog(ExportDialog.this, true); 
					g.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE); 
					g.setLocationRelativeTo(null);
					g.setSize(400, 500); 
					JTextArea at = new JTextArea(writer.getBuffer().toString()); 
					at.setLineWrap(true); 
					g.add(at); 
					g.setVisible(true);
				} catch (IOException ex) {
					ex.printStackTrace();
					JOptionPane.showMessageDialog(ExportDialog.this, ex.getClass().getSimpleName() + ": " + ex.getMessage());
				}
			}
		});
		exportToFile.addActionListener(new ActionListener() {
			JFileChooser fc = new JFileChooser(currentDirectory);
			
			@Override
			public void actionPerformed(ActionEvent e) {
				BoardData boardData = getBoardData();
				
				int returnVal = fc.showSaveDialog(ExportDialog.this);
				currentDirectory = fc.getCurrentDirectory();
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile(); 
					try {
						SquareIO.save(boardData, new FileWriter(file)); 
					} catch (FileNotFoundException e1) {
						e1.printStackTrace();
						JOptionPane.showMessageDialog(ExportDialog.this, "File not found");
					} catch (IOException e1) {
						e1.printStackTrace();
						JOptionPane.showMessageDialog(ExportDialog.this, e1.getClass().getSimpleName() + ": " + e1.getMessage());
					} finally {
						ExportDialog.this.dispose();
					}
				}
			}
		});
	}

	private BoardData getBoardData() {
		List<String> commentLines = comments.getText().length() > 0 ? Arrays.asList(comments.getText().split("\n")) : null;
		Board b = boardPanel.getSelectedBoard();
		String overrideRule = rule.getText();
		if (overrideRule.isEmpty())
			overrideRule = null;
		return new BoardData(b, name.getText(), creator.getText(), overrideRule, commentLines);
	}
	public File getCurrentDirectory() {
		return currentDirectory;
	}
	public static void main(String[] args) {
		Board board = new BoardImpl();
		board.addCell(GameOfLife.ALIVE, 0, 0);
		board.addCell(GameOfLife.ALIVE, 1, 1);
		board.addCell(GameOfLife.ALIVE, 1, 2);
		board.addCell(GameOfLife.ALIVE, 0, 2);
		board.addCell(GameOfLife.ALIVE, -1, 2);
		new ExportDialog(null, board, new File("."));
	}
}
