package org.bitbucket.zetro.squaredino.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.lang.ref.WeakReference;
import java.util.Scanner;
import javax.swing.BorderFactory;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.LineBorder;
import org.bitbucket.zetro.squaredino.backend.Board;
import org.bitbucket.zetro.squaredino.backend.BoardFactory;
import org.bitbucket.zetro.squaredino.io.BoardData;
import org.bitbucket.zetro.squaredino.io.SquareIO;

public class ImportDialog extends JDialog {
	/**
	 * 
	 */
	private static final long serialVersionUID = -7064813701657668775L;

	private File currentDirectory;
	
	private JTextArea text; 
	private JButton importt; 
	private JButton open; 
	
	private JLabel name = new JLabel("Name: "); 
	private JLabel creator = new JLabel("Creator: "); 
	private JLabel rule = new JLabel("Rule: "); 
	private JLabel width = new JLabel("Width/Height: "); 
	
	private BoardPanel bp;
	private BoardData importData;
	private WeakReference<Board> panelBoard;
	
	public ImportDialog(JFrame owner, File currentDirectory) {
		super(owner, true);
		
		this.currentDirectory = currentDirectory;
		
		setTitle("Import"); 
		setSize(600, 500); 
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE); 
		setLocationRelativeTo(null); 
		
		Box wrap = Box.createVerticalBox(); 
		wrap.add(createTextAndButtons()); 
		wrap.add(Box.createVerticalStrut(24)); 
		wrap.add(createInfoAndBoard()); 
		wrap.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		add(wrap); 
		setVisible(true); 
	}
	private JPanel createTextAndButtons() {
		JPanel textAndButtons = new JPanel(); 
		textAndButtons.setSize(new Dimension(600, 200)); 
		textAndButtons.setPreferredSize(new Dimension(600, 200)); 
		
		textAndButtons.setLayout(new BorderLayout()); 
		
		text = new JTextArea(); 
		textAndButtons.add(new JScrollPane(text), BorderLayout.CENTER);
		
		JPanel buttons = new JPanel(); 
		buttons.setLayout(new BoxLayout(buttons, BoxLayout.LINE_AXIS));  
		buttons.add(Box.createHorizontalGlue()); 
		
		open = new JButton("Open file..."); 
		open.setFont(new Font("DejaVu", Font.PLAIN, 11)); 
		open.addActionListener(new ActionListener() {
			JFileChooser fileChooser = new JFileChooser(currentDirectory);
			
			@Override
			public void actionPerformed(ActionEvent e) {
				int returnVal = fileChooser.showOpenDialog(ImportDialog.this);
				currentDirectory = fileChooser.getCurrentDirectory();
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fileChooser.getSelectedFile();

					try {
						loadFile(file);
					} catch (IOException ex) {
						JOptionPane.showMessageDialog(ImportDialog.this, ex.getClass().getSimpleName() + ": " + ex.getMessage());
						ex.printStackTrace();
					}
				}
			}
		});
		buttons.add(open); 
		
		buttons.add(Box.createHorizontalStrut(32)); 
		
		importt = new JButton("Import pattern");
		importt.setFont(new Font("DejaVu", Font.PLAIN, 11)); 
		importt.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					ImportDialog.this.importData = SquareIO.load(new StringReader(text.getText()));
				} catch (IOException e1) {
					JOptionPane.showMessageDialog(ImportDialog.this, e1.getClass().getSimpleName() + ": " + e1.getMessage());
					e1.printStackTrace();
				}
				dispose();
			}
		});
		buttons.add(importt); 
		
		Box wrap = new Box(BoxLayout.PAGE_AXIS);
		wrap.add(Box.createVerticalStrut(8));
		wrap.add(buttons);
		textAndButtons.add(wrap, BorderLayout.PAGE_END); 
		return textAndButtons; 
	}
	private JPanel createInfoAndBoard() {
		JPanel infoAndBoard = new JPanel();
		infoAndBoard.setLayout(new BoxLayout(infoAndBoard, BoxLayout.LINE_AXIS));
		
		/* Adds the labels to a Box */
		Box info = Box.createVerticalBox(); 
		info.setPreferredSize(new Dimension(200, 200)); 
		info.add(Box.createVerticalGlue()); 
		info.add(addJLabel(name)); 
		info.add(addJLabel(creator)); 
		info.add(addJLabel(rule)); 
		info.add(addJLabel(width)); 
		info.add(Box.createVerticalGlue()); 
		
		infoAndBoard.add(Box.createHorizontalGlue());
		infoAndBoard.add(info); 
		infoAndBoard.add(Box.createHorizontalStrut(48)); 
		
		bp = new BoardPanel(0, 0); 
		bp.setEditable(false);
		bp.setBorder(LineBorder.createGrayLineBorder());
//		bp.setSize(new Dimension(400, 300)); 
		bp.setPreferredSize(new Dimension(300, 200));
//		bp.setMaximumSize(new Dimension(300, 200));
//		bp.setBounds(new Rectangle(400, 300));
		infoAndBoard.add(bp); 
		
		return infoAndBoard; 
	}
	private JLabel addJLabel(JLabel l) {
		l.setAlignmentX(Box.LEFT_ALIGNMENT); 
		l.setFont(new Font("DejaVu", Font.PLAIN, 11)); 
		return l; 
	}
	private void loadFile(File file) throws FileNotFoundException, IOException {
		Scanner scanner = new Scanner(file);
		StringBuilder s = new StringBuilder();
		
		while(scanner.hasNext()) {
			s.append(scanner.nextLine()).append('\n');
		}
		
		text.setText(s.toString());
		loadBoard(new StringReader(s.toString()));
	}
	private void loadBoard(Reader reader) throws IOException {
		BoardData boardData = SquareIO.load(reader);

		updateInfo(boardData);
		if (boardData.getBoard() != null) {
			panelBoard = new WeakReference<Board>(boardData.getBoard());
			bp.setBoard(BoardFactory.unmodifiableBoard(panelBoard.get()));
		}
	}
	private void updateInfo(BoardData boardData) {
		if (boardData.getName() != null) {
			name.setText("Name: " + boardData.getName());
		} else {
			name.setText("Name: ");
		}
		if (boardData.getAuthor() != null) {
			creator.setText("Creator: " + boardData.getAuthor());
		} else {
			creator.setText("Creator: ");
		}
		Board board = boardData.getBoard();
		if (board == null) {
			System.err.println("Error: Board failed to load!");
			JOptionPane.showMessageDialog(this, "Error: Board failed to load");
		} else {
			rule.setText("Rule: " + board.getRuleAsString());
			String sw = boardData.getWidth() == Board.INFINITE ? "-" : String.valueOf(boardData.getWidth());
			String sh = boardData.getHeight() == Board.INFINITE ? "-" : String.valueOf(boardData.getHeight());
			width.setText("Width/Height: " + sw + "/" + sh);
		}
	}

	/**
	 * Clears all imported board data.
	 */
	public void clear() {
		importData = null;
		if (panelBoard != null) {
			Board board = panelBoard.get();
			if (board != null) {
				board.clear(); // Note: board != importData.getBoard()
			}
		}
	}
	
	public BoardData getBoardData() {
		return importData;
	}
	public File getCurrentDirectory() {
		return currentDirectory;
	}
	public static void main(String[] args) {
		new ImportDialog(null, new File(".")); 
	}
}
