package org.bitbucket.zetro.squaredino.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import org.bitbucket.zetro.squaredino.backend.BoardFactory;
import org.bitbucket.zetro.squaredino.backend.experimental.HashAutomaton;

public class SquareDinoWindow extends JFrame {
	
	private static final long serialVersionUID = -8197485810746004708L;

	private BoardPanel bp; 
	private ControlBar control; 
	private GraphPanel gp; 
	public SquareDinoWindow() {
		super("SquareDino"); 
		setSize(800, 600); 
		setDefaultCloseOperation(EXIT_ON_CLOSE); 
		setLocationRelativeTo(null); 
		
		bp = new BoardPanel(50, 50); 
		
		setLayout(new BorderLayout());
		add(bp, BorderLayout.CENTER); 
		
		gp = new GraphPanel(); 
		add(gp, BorderLayout.PAGE_END);
		
		control = new ControlBar(this, bp, gp); 
		add(control, BorderLayout.PAGE_START); 
		
		addWindowFocusListener(new WindowAdapter() {
			@Override
			public void windowGainedFocus(WindowEvent e) {
				bp.requestFocusInWindow();
			}
		});
		
		add(new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				System.out.println("Standard");
				BoardFactory.useExperimental = false;
			}
		}, KeyEvent.VK_1);
		add(new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				System.out.println("Linear Hashlife");
				BoardFactory.useExperimental = true;
				HashAutomaton.useLinear = true;
			}
		}, KeyEvent.VK_2);
		add(new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				System.out.println("Superspeed Hashlife");
				BoardFactory.useExperimental = true;
				HashAutomaton.useLinear = false;
			}
		}, KeyEvent.VK_3);
		add(new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				HashAutomaton.stepSize = Math.min(20, HashAutomaton.stepSize + 1);
				System.out.println("Step size: " + (1 << HashAutomaton.stepSize));
			}
		}, KeyEvent.VK_ADD);
		add(new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				HashAutomaton.stepSize = Math.max(0, HashAutomaton.stepSize - 1);
				System.out.println("Step size: " + (1 << HashAutomaton.stepSize));
			}
		}, KeyEvent.VK_SUBTRACT);
	}

	private void add(Action action, int key) {
		KeyStroke keyStroke = KeyStroke.getKeyStroke(key, 0);
		getRootPane().getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(keyStroke, key);
		getRootPane().getActionMap().put(key, action);
	}
}
