package org.bitbucket.zetro.squaredino.io;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import org.bitbucket.zetro.squaredino.backend.Board;
import org.bitbucket.zetro.squaredino.backend.Cell;
import org.bitbucket.zetro.squaredino.misc.Point;

/**
 * Data class containing a <code>Board</code> and data about it.
 */
public class BoardData {
	private Board board;
	private String name, author, rule;
	private List<String> comments;
	private int width, height;

	public BoardData(Board board, String name, String author, String overrideRule, List<String> comments) {
		if (board == null) {
			throw new NullPointerException();
		}

		this.board = board;
		this.name = name;
		this.author = author;
		this.rule = overrideRule;
		this.comments = comments;

		calculateDimension();
	}

	public BoardData(Board board, String name, String author, List<String> comments, int width, int height) {
		this.board = board;
		this.name = name;
		this.author = author;
		this.comments = comments;
		this.width = width;
		this.height = height;
	}

	private void calculateDimension() {
		int lowX = Integer.MAX_VALUE;
		int highX = Integer.MIN_VALUE;
		int lowY = Integer.MAX_VALUE;
		int highY = Integer.MIN_VALUE;

		if (board.getNumberOfCells() > 0) {
			for (Map.Entry<Point, Cell> entry : board) {
				Point p = entry.getKey();

				if (p.x < lowX) {
					lowX = p.x;
				}
				if (p.x > highX) {
					highX = p.x;
				}
				if (p.y < lowY) {
					lowY = p.y;
				}
				if (p.y > highY) {
					highY = p.y;
				}
			}

			this.width = highX - lowX + 1;
			this.height = highY - lowY + 1;
		}
	}

	public Board getBoard() {
		return board;
	}

	public String getName() {
		return name;
	}

	public String getAuthor() {
		return author;
	}

	public String getRule() {
		return rule == null ? board.getRuleAsString() : rule;
	}

	public List<String> getComments() {
		return comments != null ? Collections.unmodifiableList(comments) : null;
	}

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}
}
