package org.bitbucket.zetro.squaredino.misc;


/**
 * A class that represents a rectangle, optimal for
 * drawing with Graphics. 
 *
 */
public class Selection {
	private Point p1, p2; 
	private Point topLeft; 
	private Point bottomRight; 
	
	/**
	 * Creates a rectangle with the point p as all corners. 
	 * @param p The corner. 
	 */
	public Selection(Point p) {
		p1 = p2 = p; 
		calculate(); 
	}
	/**
	 * Creates a rectangle between p and t. 
	 * @param p The first point 
	 * @param t The second point 
	 */
	public Selection(Point p1, Point p2) {
		this.p1 = p1; 
		this.p2 = p2; 
		calculate(); 
	}
	private void calculate() {
		/* Stupid things because I can't draw with negative width/height */
		if(p1.x <= p2.x && p1.y <= p2.y) {
			topLeft = p1; 
			bottomRight = p2; 
		}
		else if(p1.x <= p2.x && p1.y >= p2.y) {
			topLeft = new Point(p1.x, p2.y);
			bottomRight = new Point(p2.x, p1.y);
		}
		else if(p1.x >= p2.x && p1.y <= p2.y) {
			topLeft = new Point(p2.x, p1.y);
			bottomRight = new Point(p1.x, p2.y); 
		}
		else if(p1.x >= p2.y && p1.y >= p2.y) {
			topLeft = p2; 
			bottomRight = p1; 
		}
		else
			System.err.println("Error in Selection.java"); 
	}
	/**
	 * Returns the top left corner. 
	 * 
	 * @return the top left corner 
	 */
	public Point getTopLeft() {
		return topLeft; 
	}
	
	/**
	 * Sets the first point. 
	 * @param p the first point
	 */
	public void setFirst(Point p) {
		p1 = p; 
		calculate(); 
	}
	/**
	 * Returns the bottom right corner. 
	 * @return the bottom right corner. 
	 */
	public Point getBottomRight() {
		return bottomRight; 
	}
	
	public boolean contains(Point p) {
		return (p.x >= getTopLeft().x && p.x <= getBottomRight().x &&
				p.y >= getTopLeft().y && p.y <= getBottomRight().y); 
	}
	
	public void moveX(int x) {
		p1 = new Point(p1.x+x, p1.y); 
		p2 = new Point(p2.x+x, p2.y); 
		calculate();  
	}
	public void moveY(int y) {
		p1 = new Point(p1.x, p1.y+y);
		p2 = new Point(p2.x, p2.y+y); 
		calculate(); 
	}
	public void move(Point p) {
		p1 = new Point(p1.x + p.x, p1.y + p.y);  
		p2 = new Point(p2.x + p.x, p2.y + p.y); 
		calculate(); 
	}
	/**
	 * Sets the last point. 
	 * @param p the last point 
	 */
	public void setLast(Point p) {
		p2 = p; 
		calculate(); 
	}
	
	/**
	 * Returns the (absolute) width of the rectangle. 
	 * @return the width of the rectangle 
	 */
	public int getWidth() {
		return Math.abs(bottomRight.x - topLeft.x); 
	}
	/**
	 * Returns the (absolute) height of the rectangle. 
	 * @return the height of the rectangle 
	 */
	public int getHeight() {
		return Math.abs(bottomRight.y - topLeft.y);  
	}
}
